/*
 * Copyright (c) 2017, Red Hat Inc. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.gcbench.tests;

import org.openjdk.jmh.runner.options.Options;
import org.openjdk.jmh.util.Utils;

import java.io.PrintWriter;
import java.util.concurrent.TimeUnit;

public abstract class AbstractTest implements Test {

    protected final PrintWriter pw;
    protected final Options baseOpts;
    public final Class<?> benchmark;
    protected final String label;
    protected final String description;

    public AbstractTest(Options baseOpts, String label, Class<?> benchmark, String description) {
        this.baseOpts = baseOpts;
        this.label = label;
        this.benchmark = benchmark;
        this.description = description;
        this.pw = new PrintWriter(System.out, true);
    }

    @Override
    public String label() {
        return label;
    }

    @Override
    public String description() {
        return description;
    }

    @Override
    public final void run() {
        pw.println("===== " + label);
        pw.println();

        Utils.reflow(pw,
                description,
                80, 2);

        pw.println();

        long time1 = System.nanoTime();

        doRun();

        long time2 = System.nanoTime();

        pw.println();
        pw.println("  Test took " + TimeUnit.NANOSECONDS.toSeconds(time2 - time1) + " seconds.");
        pw.println();
    }

    protected abstract void doRun();
}
